@@ -1,21 +1,7 @@
 /*
- * Copyright (C) 2019 Mikhail Zolotukhin <zomial@protonmail.com>
+ * SPDX-FileCopyrightText: 2019 Mikhail Zolotukhin <zomial@protonmail.com>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of
- * the License or (at your option) version 3 or any later version
- * accepted by the membership of KDE e.V. (or its successor approved
- * by the membership of KDE e.V.), which shall act as a proxy
- * defined in Section 14 of version 3 of the license.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
 #include <QFont>
@@ -23,6 +9,8 @@
 #include <QColor>
 #include <QMap>
 #include <QList>
+#include <QSvgGenerator>
+#include <QDir>
 
 #include <KConfig>
 #include <KSharedConfig>
@@ -32,6 +20,7 @@
 
 #include <gtk/gtk.h>
 
+#include "decorationpainter.h"
 #include "configvalueprovider.h"
 
 ConfigValueProvider::ConfigValueProvider() :
@@ -128,6 +117,12 @@ QString ConfigValueProvider::cursorThemeName() const
     return configGroup.readEntry(QStringLiteral("cursorTheme"), QStringLiteral("breeze_cursors"));
 }
 
+int ConfigValueProvider::cursorSize() const
+{
+    KConfigGroup configGroup = inputConfig->group(QStringLiteral("Mouse"));
+    return configGroup.readEntry(QStringLiteral("cursorSize"), 24);
+}
+
 bool ConfigValueProvider::iconsOnButtons() const
 {
     KConfigGroup configGroup = kdeglobalsConfig->group(QStringLiteral("KDE"));
@@ -173,6 +168,54 @@ bool ConfigValueProvider::preferDarkTheme() const
     return windowBackgroundGray < 192;
 }
 
+QStringList ConfigValueProvider::windowDecorationsButtonsImages() const
+{
+    static const QVector<QString> buttonTypes {
+        QStringLiteral("close"),
+        QStringLiteral("maximize"),
+        QStringLiteral("maximized"),
+        QStringLiteral("minimize"),
+    };
+
+    static const QVector<QString> buttonStates {
+        // Focused titlebars
+        QStringLiteral("normal"),
+        QStringLiteral("active"), // aka pressed
+        QStringLiteral("hover"),
+        // Unfocused titlebars
+        QStringLiteral("backdrop-normal"),
+        QStringLiteral("backdrop-active"),
+        QStringLiteral("backdrop-hover"),
+    };
+
+    KConfigGroup decorationGroup = kwinConfig->group(QStringLiteral("org.kde.kdecoration2"));
+    const QString themeName = decorationGroup.readEntry(QStringLiteral("theme"), QStringLiteral("Breeze"));
+
+    auto decorationPainter = DecorationPainter::fromThemeName(themeName);
+    QStringList decorationsImages {};
+
+    for (const auto &buttonType : buttonTypes) {
+        for (const auto &buttonState : buttonStates) {
+            QSvgGenerator svgGenerator {};
+
+            const QString fileDirPath = QDir::tempPath() + QStringLiteral("/plasma-csd-generator");
+            QDir(fileDirPath).mkpath(QStringLiteral("."));
+            QString filePath = QStringLiteral("%1/%2-%3.svg").arg(fileDirPath, buttonType, buttonState);
+
+            svgGenerator.setFileName(filePath);
+            svgGenerator.setViewBox(DecorationPainter::ButtonGeometry);
+
+            QPainter painter {&svgGenerator};
+            decorationPainter->paintButton(painter, buttonType, buttonState);
+            painter.end();
+
+            decorationsImages.append(filePath);
+        }
+    }
+
+    return decorationsImages;
+}
+
 QString ConfigValueProvider::windowDecorationsButtonsOrder() const
 {
     KConfigGroup configGroup = kwinConfig->group(QStringLiteral("org.kde.kdecoration2"));
