@@ -1,22 +1,8 @@
 /*
- * Copyright (C) 2019 Mikhail Zolotukhin <zomial@protonmail.com>
- * Copyright (C) 2019 Nicolas Fella <nicolas.fella@gmx.de>
+ * SPDX-FileCopyrightText: 2019 Mikhail Zolotukhin <zomial@protonmail.com>
+ * SPDX-FileCopyrightText: 2019 Nicolas Fella <nicolas.fella@gmx.de>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of
- * the License or (at your option) version 3 or any later version
- * accepted by the membership of KDE e.V. (or its successor approved
- * by the membership of KDE e.V.), which shall act as a proxy
- * defined in Section 14 of version 3 of the license.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
 #include <QFont>
@@ -40,7 +26,8 @@ GtkConfig::GtkConfig(QObject *parent, const QVariantList&) :
     themePreviewer(new ThemePreviewer(this)),
     kdeglobalsConfigWatcher(KConfigWatcher::create(KSharedConfig::openConfig(QStringLiteral("kdeglobals")))),
     kwinConfigWatcher(KConfigWatcher::create(KSharedConfig::openConfig(QStringLiteral("kwinrc")))),
-    kcminputConfigWatcher(KConfigWatcher::create(KSharedConfig::openConfig(QStringLiteral("kcminputrc"))))
+    kcminputConfigWatcher(KConfigWatcher::create(KSharedConfig::openConfig(QStringLiteral("kcminputrc")))),
+    breezeConfigWatcher(KConfigWatcher::create(KSharedConfig::openConfig(QStringLiteral("breezerc"))))
 {
     QDBusConnection dbus = QDBusConnection::sessionBus();
     dbus.registerService(QStringLiteral("org.kde.GtkConfig"));
@@ -49,6 +36,7 @@ GtkConfig::GtkConfig(QObject *parent, const QVariantList&) :
     connect(kdeglobalsConfigWatcher.data(), &KConfigWatcher::configChanged, this, &GtkConfig::onKdeglobalsSettingsChange);
     connect(kwinConfigWatcher.data(), &KConfigWatcher::configChanged, this, &GtkConfig::onKWinSettingsChange);
     connect(kcminputConfigWatcher.data(), &KConfigWatcher::configChanged, this, &GtkConfig::onKCMInputSettingsChange);
+    connect(breezeConfigWatcher.data(), &KConfigWatcher::configChanged, this, &GtkConfig::onBreezeSettingsChange);
 
     ConfigEditor::removeLegacyGtk2Strings();
     applyAllSettings();
@@ -61,34 +49,23 @@ GtkConfig::~GtkConfig()
     dbus.unregisterObject(QStringLiteral("/GtkConfig"));
 }
 
-void GtkConfig::setGtk2Theme(const QString &themeName) const
+void GtkConfig::setGtkTheme(const QString &themeName) const
 {
     ConfigEditor::setGtk2ConfigValue(QStringLiteral("gtk-theme-name"), themeName);
-}
-
-void GtkConfig::setGtk3Theme(const QString &themeName) const
-{
     ConfigEditor::setGtk3ConfigValueGSettings(QStringLiteral("gtk-theme"), themeName);
     ConfigEditor::setGtk3ConfigValueSettingsIni(QStringLiteral("gtk-theme-name"), themeName);
     ConfigEditor::setGtk3ConfigValueXSettingsd(QStringLiteral("Net/ThemeName"),  themeName);
+
+    // Window decorations are part of the theme, in case of Breeze we inject custom ones from KWin
+    setWindowDecorationsAppearance();
 }
 
-QString GtkConfig::gtk2Theme() const
-{
-    return ConfigEditor::gtk2ConfigValue(QStringLiteral("gtk-theme-name"));
-}
-
-QString GtkConfig::gtk3Theme() const
+QString GtkConfig::gtkTheme() const
 {
     return ConfigEditor::gtk3ConfigValueSettingsIni(QStringLiteral("gtk-theme-name"));
 }
 
-void GtkConfig::showGtk2ThemePreview(const QString& themeName) const
-{
-    themePreviewer->showGtk2App(themeName);
-}
-
-void GtkConfig::showGtk3ThemePreview(const QString& themeName) const
+void GtkConfig::showGtkThemePreview(const QString& themeName) const
 {
     themePreviewer->showGtk3App(themeName);
 }
@@ -120,6 +97,15 @@ void GtkConfig::setCursorTheme() const
     ConfigEditor::setGtk3ConfigValueXSettingsd(QStringLiteral("Gtk/CursorThemeName"),  cursorThemeName);
 }
 
+void GtkConfig::setCursorSize() const
+{
+    const int cursorSize = configValueProvider->cursorSize();
+    ConfigEditor::setGtk2ConfigValue(QStringLiteral("gtk-cursor-theme-size"), cursorSize);
+    ConfigEditor::setGtk3ConfigValueGSettings(QStringLiteral("cursor-size"), cursorSize);
+    ConfigEditor::setGtk3ConfigValueSettingsIni(QStringLiteral("gtk-cursor-theme-size"), cursorSize);
+    ConfigEditor::setGtk3ConfigValueXSettingsd(QStringLiteral("Gtk/CursorThemeSize"),  cursorSize);
+}
+
 void GtkConfig::setIconsOnButtons() const
 {
     const bool iconsOnButtonsConfigValue = configValueProvider->iconsOnButtons();
@@ -159,6 +145,16 @@ void GtkConfig::setDarkThemePreference() const
     ConfigEditor::setGtk3ConfigValueSettingsIni(QStringLiteral("gtk-application-prefer-dark-theme"), preferDarkTheme);
 }
 
+void GtkConfig::setWindowDecorationsAppearance() const
+{
+    if (gtkTheme() == QStringLiteral("Breeze")) { // Only Breeze GTK supports custom decoration buttons
+        const auto windowDecorationsButtonsImages = configValueProvider->windowDecorationsButtonsImages();
+        ConfigEditor::setCustomClientSideDecorations(windowDecorationsButtonsImages);
+    } else {
+        ConfigEditor::disableCustomClientSideDecorations();
+    }
+}
+
 void GtkConfig::setWindowDecorationsButtonsOrder() const
 {
     const QString windowDecorationsButtonOrder = configValueProvider->windowDecorationsButtonsOrder();
@@ -187,11 +183,13 @@ void GtkConfig::applyAllSettings() const
     setFont();
     setIconTheme();
     setCursorTheme();
+    setCursorSize();
     setIconsOnButtons();
     setIconsInMenus();
     setToolbarStyle();
     setScrollbarBehavior();
     setDarkThemePreference();
+    setWindowDecorationsAppearance();
     setWindowDecorationsButtonsOrder();
     setEnableAnimations();
     setColors();
@@ -223,6 +221,7 @@ void GtkConfig::onKdeglobalsSettingsChange(const KConfigGroup &group, const QByt
         if (names.contains(QByteArrayLiteral("ColorScheme"))) {
             setColors();
             setDarkThemePreference();
+            setWindowDecorationsAppearance(); // Decorations' color can depend on the current color scheme
         }
     } else if (group.name() == QStringLiteral("Toolbar style")) {
         if (names.contains(QByteArrayLiteral("ToolButtonStyle"))) {
@@ -233,17 +232,34 @@ void GtkConfig::onKdeglobalsSettingsChange(const KConfigGroup &group, const QByt
 
 void GtkConfig::onKWinSettingsChange(const KConfigGroup &group, const QByteArrayList &names) const
 {
-    if (group.name() == QStringLiteral("org.kde.kdecoration2")
-            && (names.contains("ButtonsOnRight") || names.contains("ButtonsOnLeft"))) {
-        setWindowDecorationsButtonsOrder();
+    if (group.name() == QStringLiteral("org.kde.kdecoration2")) {
+        if (names.contains(QByteArrayLiteral("ButtonsOnRight"))
+            || names.contains(QByteArrayLiteral("ButtonsOnLeft"))) {
+            setWindowDecorationsButtonsOrder();
+        }
+        if (names.contains(QByteArrayLiteral("theme"))) {
+            setWindowDecorationsAppearance();
+        }
     }
 }
 
 void GtkConfig::onKCMInputSettingsChange(const KConfigGroup& group, const QByteArrayList& names) const
 {
-    if (group.name() == QStringLiteral("Mouse")
-            && names.contains("cursorTheme")) {
-        setCursorTheme();
+    if (group.name() == QStringLiteral("Mouse")) {
+        if (names.contains("cursorTheme")) {
+            setCursorTheme();
+        }
+        if (names.contains("cursorSize")) {
+            setCursorSize();
+        }
+    }
+}
+
+void GtkConfig::onBreezeSettingsChange(const KConfigGroup& group, const QByteArrayList& names) const
+{
+    if (group.name() == QStringLiteral("Common")
+            && names.contains("OutlineCloseButton")) {
+        setWindowDecorationsAppearance();
     }
 }
 
